
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __org_omg_PortableServer_POAPackage_ObjectAlreadyActive__
#define __org_omg_PortableServer_POAPackage_ObjectAlreadyActive__

#pragma interface

#include <org/omg/CORBA/UserException.h>
extern "Java"
{
  namespace org
  {
    namespace omg
    {
      namespace PortableServer
      {
        namespace POAPackage
        {
            class ObjectAlreadyActive;
        }
      }
    }
  }
}

class org::omg::PortableServer::POAPackage::ObjectAlreadyActive : public ::org::omg::CORBA::UserException
{

public:
  ObjectAlreadyActive();
  ObjectAlreadyActive(::java::lang::String *);
private:
  static const jlong serialVersionUID = -2919308871345098101LL;
public:
  static ::java::lang::Class class$;
};

#endif // __org_omg_PortableServer_POAPackage_ObjectAlreadyActive__
