
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_text_AbstractDocument$AttributeContext__
#define __javax_swing_text_AbstractDocument$AttributeContext__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace javax
  {
    namespace swing
    {
      namespace text
      {
          class AbstractDocument$AttributeContext;
          class AttributeSet;
      }
    }
  }
}

class javax::swing::text::AbstractDocument$AttributeContext : public ::java::lang::Object
{

public:
  virtual ::javax::swing::text::AttributeSet * addAttribute(::javax::swing::text::AttributeSet *, ::java::lang::Object *, ::java::lang::Object *) = 0;
  virtual ::javax::swing::text::AttributeSet * addAttributes(::javax::swing::text::AttributeSet *, ::javax::swing::text::AttributeSet *) = 0;
  virtual ::javax::swing::text::AttributeSet * getEmptySet() = 0;
  virtual void reclaim(::javax::swing::text::AttributeSet *) = 0;
  virtual ::javax::swing::text::AttributeSet * removeAttribute(::javax::swing::text::AttributeSet *, ::java::lang::Object *) = 0;
  virtual ::javax::swing::text::AttributeSet * removeAttributes(::javax::swing::text::AttributeSet *, ::javax::swing::text::AttributeSet *) = 0;
  virtual ::javax::swing::text::AttributeSet * removeAttributes(::javax::swing::text::AttributeSet *, ::java::util::Enumeration *) = 0;
  static ::java::lang::Class class$;
} __attribute__ ((java_interface));

#endif // __javax_swing_text_AbstractDocument$AttributeContext__
