
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_security_auth_login_CredentialNotFoundException__
#define __javax_security_auth_login_CredentialNotFoundException__

#pragma interface

#include <javax/security/auth/login/CredentialException.h>
extern "Java"
{
  namespace javax
  {
    namespace security
    {
      namespace auth
      {
        namespace login
        {
            class CredentialNotFoundException;
        }
      }
    }
  }
}

class javax::security::auth::login::CredentialNotFoundException : public ::javax::security::auth::login::CredentialException
{

public:
  CredentialNotFoundException();
  CredentialNotFoundException(::java::lang::String *);
private:
  static const jlong serialVersionUID = -7779934467214319475LL;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_security_auth_login_CredentialNotFoundException__
