
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_naming_NoPermissionException__
#define __javax_naming_NoPermissionException__

#pragma interface

#include <javax/naming/NamingSecurityException.h>
extern "Java"
{
  namespace javax
  {
    namespace naming
    {
        class NoPermissionException;
    }
  }
}

class javax::naming::NoPermissionException : public ::javax::naming::NamingSecurityException
{

public:
  NoPermissionException();
  NoPermissionException(::java::lang::String *);
private:
  static const jlong serialVersionUID = 8395332708699751775LL;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_naming_NoPermissionException__
