
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_plaf_metal_MetalRootPaneUI$MetalTitlePane$PaneButton__
#define __javax_swing_plaf_metal_MetalRootPaneUI$MetalTitlePane$PaneButton__

#pragma interface

#include <javax/swing/JButton.h>
extern "Java"
{
  namespace javax
  {
    namespace swing
    {
        class Action;
      namespace plaf
      {
        namespace metal
        {
            class MetalRootPaneUI$MetalTitlePane;
            class MetalRootPaneUI$MetalTitlePane$PaneButton;
        }
      }
    }
  }
}

class javax::swing::plaf::metal::MetalRootPaneUI$MetalTitlePane$PaneButton : public ::javax::swing::JButton
{

public:
  MetalRootPaneUI$MetalTitlePane$PaneButton(::javax::swing::plaf::metal::MetalRootPaneUI$MetalTitlePane *, ::javax::swing::Action *);
  virtual jboolean isFocusable();
public: // actually package-private
  ::javax::swing::plaf::metal::MetalRootPaneUI$MetalTitlePane * __attribute__((aligned(__alignof__( ::javax::swing::JButton)))) this$1;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_swing_plaf_metal_MetalRootPaneUI$MetalTitlePane$PaneButton__
