
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_print_attribute_standard_Media__
#define __javax_print_attribute_standard_Media__

#pragma interface

#include <javax/print/attribute/EnumSyntax.h>
extern "Java"
{
  namespace javax
  {
    namespace print
    {
      namespace attribute
      {
        namespace standard
        {
            class Media;
        }
      }
    }
  }
}

class javax::print::attribute::standard::Media : public ::javax::print::attribute::EnumSyntax
{

public: // actually protected
  Media(jint);
public:
  virtual jboolean equals(::java::lang::Object *);
  virtual ::java::lang::Class * getCategory();
  virtual ::java::lang::String * getName();
private:
  static const jlong serialVersionUID = -2823970704630722439LL;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_print_attribute_standard_Media__
