
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __org_omg_CORBA_DynAnyPackage_InvalidValue__
#define __org_omg_CORBA_DynAnyPackage_InvalidValue__

#pragma interface

#include <org/omg/CORBA/UserException.h>
extern "Java"
{
  namespace org
  {
    namespace omg
    {
      namespace CORBA
      {
        namespace DynAnyPackage
        {
            class InvalidValue;
        }
      }
    }
  }
}

class org::omg::CORBA::DynAnyPackage::InvalidValue : public ::org::omg::CORBA::UserException
{

public:
  InvalidValue();
  InvalidValue(::java::lang::String *);
private:
  static const jlong serialVersionUID = -4741963043279146697LL;
public:
  static ::java::lang::Class class$;
};

#endif // __org_omg_CORBA_DynAnyPackage_InvalidValue__
