
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_plaf_basic_BasicTableHeaderUI$1__
#define __javax_swing_plaf_basic_BasicTableHeaderUI$1__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace java
  {
    namespace awt
    {
      namespace event
      {
          class ActionEvent;
      }
    }
  }
  namespace javax
  {
    namespace swing
    {
      namespace plaf
      {
        namespace basic
        {
            class BasicTableHeaderUI$1;
            class BasicTableHeaderUI$MouseInputHandler;
        }
      }
    }
  }
}

class javax::swing::plaf::basic::BasicTableHeaderUI$1 : public ::java::lang::Object
{

public: // actually package-private
  BasicTableHeaderUI$1(::javax::swing::plaf::basic::BasicTableHeaderUI$MouseInputHandler *);
public:
  virtual void actionPerformed(::java::awt::event::ActionEvent *);
public: // actually package-private
  ::javax::swing::plaf::basic::BasicTableHeaderUI$MouseInputHandler * __attribute__((aligned(__alignof__( ::java::lang::Object)))) this$1;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_swing_plaf_basic_BasicTableHeaderUI$1__
