
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_net_ssl_KeyManagerFactory$1__
#define __javax_net_ssl_KeyManagerFactory$1__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace javax
  {
    namespace net
    {
      namespace ssl
      {
          class KeyManagerFactory$1;
      }
    }
  }
}

class javax::net::ssl::KeyManagerFactory$1 : public ::java::lang::Object
{

public: // actually package-private
  KeyManagerFactory$1();
public:
  virtual ::java::lang::Object * run();
  static ::java::lang::Class class$;
};

#endif // __javax_net_ssl_KeyManagerFactory$1__
