
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __java_lang_InternalError__
#define __java_lang_InternalError__

#pragma interface

#include <java/lang/VirtualMachineError.h>

class java::lang::InternalError : public ::java::lang::VirtualMachineError
{

public:
  InternalError();
  InternalError(::java::lang::String *);
private:
  static const jlong serialVersionUID = -9062593416125562365LL;
public:
  static ::java::lang::Class class$;
};

#endif // __java_lang_InternalError__
