
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_text_StyledEditorKit$StyledViewFactory__
#define __javax_swing_text_StyledEditorKit$StyledViewFactory__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace javax
  {
    namespace swing
    {
      namespace text
      {
          class Element;
          class StyledEditorKit$StyledViewFactory;
          class View;
      }
    }
  }
}

class javax::swing::text::StyledEditorKit$StyledViewFactory : public ::java::lang::Object
{

public: // actually package-private
  StyledEditorKit$StyledViewFactory();
public:
  virtual ::javax::swing::text::View * create(::javax::swing::text::Element *);
  static ::java::lang::Class class$;
};

#endif // __javax_swing_text_StyledEditorKit$StyledViewFactory__
