
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_crypto_spec_IvParameterSpec__
#define __javax_crypto_spec_IvParameterSpec__

#pragma interface

#include <java/lang/Object.h>
#include <gcj/array.h>

extern "Java"
{
  namespace javax
  {
    namespace crypto
    {
      namespace spec
      {
          class IvParameterSpec;
      }
    }
  }
}

class javax::crypto::spec::IvParameterSpec : public ::java::lang::Object
{

public:
  IvParameterSpec(JArray< jbyte > *);
  IvParameterSpec(JArray< jbyte > *, jint, jint);
  virtual JArray< jbyte > * getIV();
private:
  JArray< jbyte > * __attribute__((aligned(__alignof__( ::java::lang::Object)))) iv;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_crypto_spec_IvParameterSpec__
