
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __org_w3c_dom_DOMErrorHandler__
#define __org_w3c_dom_DOMErrorHandler__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace org
  {
    namespace w3c
    {
      namespace dom
      {
          class DOMError;
          class DOMErrorHandler;
      }
    }
  }
}

class org::w3c::dom::DOMErrorHandler : public ::java::lang::Object
{

public:
  virtual jboolean handleError(::org::w3c::dom::DOMError *) = 0;
  static ::java::lang::Class class$;
} __attribute__ ((java_interface));

#endif // __org_w3c_dom_DOMErrorHandler__
