
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __org_omg_CORBA_BAD_POLICY__
#define __org_omg_CORBA_BAD_POLICY__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace org
  {
    namespace omg
    {
      namespace CORBA
      {
          class BAD_POLICY;
      }
    }
  }
}

class org::omg::CORBA::BAD_POLICY : public ::java::lang::Object
{

public:
  static const jshort value = 0;
  static ::java::lang::Class class$;
} __attribute__ ((java_interface));

#endif // __org_omg_CORBA_BAD_POLICY__
