
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_activity_ActivityCompletedException__
#define __javax_activity_ActivityCompletedException__

#pragma interface

#include <java/rmi/RemoteException.h>
extern "Java"
{
  namespace javax
  {
    namespace activity
    {
        class ActivityCompletedException;
    }
  }
}

class javax::activity::ActivityCompletedException : public ::java::rmi::RemoteException
{

public:
  ActivityCompletedException();
  ActivityCompletedException(::java::lang::String *);
  ActivityCompletedException(::java::lang::Throwable *);
  ActivityCompletedException(::java::lang::String *, ::java::lang::Throwable *);
  static ::java::lang::Class class$;
};

#endif // __javax_activity_ActivityCompletedException__
