
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __java_awt_datatransfer_MimeTypeParseException__
#define __java_awt_datatransfer_MimeTypeParseException__

#pragma interface

#include <java/lang/Exception.h>
extern "Java"
{
  namespace java
  {
    namespace awt
    {
      namespace datatransfer
      {
          class MimeTypeParseException;
      }
    }
  }
}

class java::awt::datatransfer::MimeTypeParseException : public ::java::lang::Exception
{

public:
  MimeTypeParseException();
  MimeTypeParseException(::java::lang::String *);
private:
  static const jlong serialVersionUID = -5604407764691570741LL;
public:
  static ::java::lang::Class class$;
};

#endif // __java_awt_datatransfer_MimeTypeParseException__
