
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_sound_midi_MidiUnavailableException__
#define __javax_sound_midi_MidiUnavailableException__

#pragma interface

#include <java/lang/Exception.h>
extern "Java"
{
  namespace javax
  {
    namespace sound
    {
      namespace midi
      {
          class MidiUnavailableException;
      }
    }
  }
}

class javax::sound::midi::MidiUnavailableException : public ::java::lang::Exception
{

public:
  MidiUnavailableException();
  MidiUnavailableException(::java::lang::String *);
private:
  static const jlong serialVersionUID = 6093809578628944323LL;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_sound_midi_MidiUnavailableException__
