
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_UIDefaults$2__
#define __javax_swing_UIDefaults$2__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace javax
  {
    namespace swing
    {
        class UIDefaults;
        class UIDefaults$2;
        class UIDefaults$ProxyLazyValue;
    }
  }
}

class javax::swing::UIDefaults$2 : public ::java::lang::Object
{

public: // actually package-private
  UIDefaults$2(::javax::swing::UIDefaults$ProxyLazyValue *, ::java::lang::String *, ::java::lang::String *);
public:
  virtual ::java::lang::Object * createValue(::javax::swing::UIDefaults *);
public: // actually package-private
  ::javax::swing::UIDefaults$ProxyLazyValue * __attribute__((aligned(__alignof__( ::java::lang::Object)))) this$1;
private:
  ::java::lang::String * val$className;
  ::java::lang::String * val$methodName;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_swing_UIDefaults$2__
