
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __org_omg_PortableInterceptor_ClientRequestInfoOperations__
#define __org_omg_PortableInterceptor_ClientRequestInfoOperations__

#pragma interface

#include <java/lang/Object.h>
#include <gcj/array.h>

extern "Java"
{
  namespace org
  {
    namespace omg
    {
      namespace CORBA
      {
          class Any;
          class Object;
          class Policy;
          class TypeCode;
      }
      namespace Dynamic
      {
          class Parameter;
      }
      namespace IOP
      {
          class ServiceContext;
          class TaggedComponent;
          class TaggedProfile;
      }
      namespace PortableInterceptor
      {
          class ClientRequestInfoOperations;
      }
    }
  }
}

class org::omg::PortableInterceptor::ClientRequestInfoOperations : public ::java::lang::Object
{

public:
  virtual ::org::omg::CORBA::Object * target() = 0;
  virtual ::org::omg::CORBA::Object * effective_target() = 0;
  virtual ::org::omg::IOP::TaggedProfile * effective_profile() = 0;
  virtual ::org::omg::IOP::TaggedComponent * get_effective_component(jint) = 0;
  virtual JArray< ::org::omg::IOP::TaggedComponent * > * get_effective_components(jint) = 0;
  virtual ::org::omg::CORBA::Policy * get_request_policy(jint) = 0;
  virtual ::java::lang::String * received_exception_id() = 0;
  virtual ::org::omg::CORBA::Any * received_exception() = 0;
  virtual void add_request_service_context(::org::omg::IOP::ServiceContext *, jboolean) = 0;
  virtual JArray< ::org::omg::Dynamic::Parameter * > * arguments() = 0;
  virtual JArray< ::java::lang::String * > * contexts() = 0;
  virtual JArray< ::org::omg::CORBA::TypeCode * > * exceptions() = 0;
  virtual ::org::omg::CORBA::Object * forward_reference() = 0;
  virtual ::org::omg::IOP::ServiceContext * get_reply_service_context(jint) = 0;
  virtual ::org::omg::IOP::ServiceContext * get_request_service_context(jint) = 0;
  virtual ::org::omg::CORBA::Any * get_slot(jint) = 0;
  virtual JArray< ::java::lang::String * > * operation_context() = 0;
  virtual ::java::lang::String * operation() = 0;
  virtual jshort reply_status() = 0;
  virtual jint request_id() = 0;
  virtual jboolean response_expected() = 0;
  virtual ::org::omg::CORBA::Any * result() = 0;
  virtual jshort sync_scope() = 0;
  static ::java::lang::Class class$;
} __attribute__ ((java_interface));

#endif // __org_omg_PortableInterceptor_ClientRequestInfoOperations__
