
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __java_util_Vector$1__
#define __java_util_Vector$1__

#pragma interface

#include <java/lang/Object.h>

class java::util::Vector$1 : public ::java::lang::Object
{

public: // actually package-private
  Vector$1(::java::util::Vector *);
public:
  virtual jboolean hasMoreElements();
  virtual ::java::lang::Object * nextElement();
private:
  jint __attribute__((aligned(__alignof__( ::java::lang::Object)))) i;
public: // actually package-private
  ::java::util::Vector * this$0;
public:
  static ::java::lang::Class class$;
};

#endif // __java_util_Vector$1__
