
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_text_ComponentView$1__
#define __javax_swing_text_ComponentView$1__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace javax
  {
    namespace swing
    {
      namespace text
      {
          class ComponentView;
          class ComponentView$1;
      }
    }
  }
}

class javax::swing::text::ComponentView$1 : public ::java::lang::Object
{

public: // actually package-private
  ComponentView$1(::javax::swing::text::ComponentView *);
public:
  virtual void run();
public: // actually package-private
  ::javax::swing::text::ComponentView * __attribute__((aligned(__alignof__( ::java::lang::Object)))) this$0;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_swing_text_ComponentView$1__
