
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_activation_MimeTypeParameterList__
#define __javax_activation_MimeTypeParameterList__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace javax
  {
    namespace activation
    {
        class MimeTypeParameterList;
    }
  }
}

class javax::activation::MimeTypeParameterList : public ::java::lang::Object
{

public:
  MimeTypeParameterList();
  MimeTypeParameterList(::java::lang::String *);
public: // actually protected
  virtual void parse(::java::lang::String *);
public:
  virtual jint size();
  virtual jboolean isEmpty();
  virtual ::java::lang::String * get(::java::lang::String *);
  virtual void set(::java::lang::String *, ::java::lang::String *);
  virtual void remove(::java::lang::String *);
  virtual ::java::util::Enumeration * getNames();
  virtual ::java::lang::String * toString();
private:
  static ::java::lang::String * quote(::java::lang::String *);
  static ::java::lang::String * unquote(::java::lang::String *);
  ::java::util::List * __attribute__((aligned(__alignof__( ::java::lang::Object)))) parameterNames;
  ::java::util::Map * parameterValues;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_activation_MimeTypeParameterList__
