
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __java_awt_dnd_InvalidDnDOperationException__
#define __java_awt_dnd_InvalidDnDOperationException__

#pragma interface

#include <java/lang/IllegalStateException.h>
extern "Java"
{
  namespace java
  {
    namespace awt
    {
      namespace dnd
      {
          class InvalidDnDOperationException;
      }
    }
  }
}

class java::awt::dnd::InvalidDnDOperationException : public ::java::lang::IllegalStateException
{

public:
  InvalidDnDOperationException();
  InvalidDnDOperationException(::java::lang::String *);
private:
  static const jlong serialVersionUID = -6062568741193956678LL;
public:
  static ::java::lang::Class class$;
};

#endif // __java_awt_dnd_InvalidDnDOperationException__
