
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_security_sasl_AuthenticationException__
#define __javax_security_sasl_AuthenticationException__

#pragma interface

#include <javax/security/sasl/SaslException.h>
extern "Java"
{
  namespace javax
  {
    namespace security
    {
      namespace sasl
      {
          class AuthenticationException;
      }
    }
  }
}

class javax::security::sasl::AuthenticationException : public ::javax::security::sasl::SaslException
{

public:
  AuthenticationException();
  AuthenticationException(::java::lang::String *);
  AuthenticationException(::java::lang::String *, ::java::lang::Throwable *);
  static ::java::lang::Class class$;
};

#endif // __javax_security_sasl_AuthenticationException__
