
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __java_awt_event_HierarchyBoundsListener__
#define __java_awt_event_HierarchyBoundsListener__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace java
  {
    namespace awt
    {
      namespace event
      {
          class HierarchyBoundsListener;
          class HierarchyEvent;
      }
    }
  }
}

class java::awt::event::HierarchyBoundsListener : public ::java::lang::Object
{

public:
  virtual void ancestorMoved(::java::awt::event::HierarchyEvent *) = 0;
  virtual void ancestorResized(::java::awt::event::HierarchyEvent *) = 0;
  static ::java::lang::Class class$;
} __attribute__ ((java_interface));

#endif // __java_awt_event_HierarchyBoundsListener__
