
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_security_sasl_RealmChoiceCallback__
#define __javax_security_sasl_RealmChoiceCallback__

#pragma interface

#include <javax/security/auth/callback/ChoiceCallback.h>
#include <gcj/array.h>

extern "Java"
{
  namespace javax
  {
    namespace security
    {
      namespace sasl
      {
          class RealmChoiceCallback;
      }
    }
  }
}

class javax::security::sasl::RealmChoiceCallback : public ::javax::security::auth::callback::ChoiceCallback
{

public:
  RealmChoiceCallback(::java::lang::String *, JArray< ::java::lang::String * > *, jint, jboolean);
  static ::java::lang::Class class$;
};

#endif // __javax_security_sasl_RealmChoiceCallback__
