
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __java_lang_management_ManagementPermission__
#define __java_lang_management_ManagementPermission__

#pragma interface

#include <java/security/BasicPermission.h>

class java::lang::management::ManagementPermission : public ::java::security::BasicPermission
{

public:
  ManagementPermission(::java::lang::String *);
  ManagementPermission(::java::lang::String *, ::java::lang::String *);
private:
  static const jlong serialVersionUID = 1897496590799378737LL;
public:
  static ::java::lang::Class class$;
};

#endif // __java_lang_management_ManagementPermission__
