
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __java_security_cert_CertPathValidator__
#define __java_security_cert_CertPathValidator__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace java
  {
    namespace security
    {
        class Provider;
      namespace cert
      {
          class CertPath;
          class CertPathParameters;
          class CertPathValidator;
          class CertPathValidatorResult;
          class CertPathValidatorSpi;
      }
    }
  }
}

class java::security::cert::CertPathValidator : public ::java::lang::Object
{

public: // actually protected
  CertPathValidator(::java::security::cert::CertPathValidatorSpi *, ::java::security::Provider *, ::java::lang::String *);
public:
  static ::java::lang::String * getDefaultType();
  static ::java::security::cert::CertPathValidator * getInstance(::java::lang::String *);
  static ::java::security::cert::CertPathValidator * getInstance(::java::lang::String *, ::java::lang::String *);
  static ::java::security::cert::CertPathValidator * getInstance(::java::lang::String *, ::java::security::Provider *);
  virtual ::java::lang::String * getAlgorithm();
  virtual ::java::security::Provider * getProvider();
  virtual ::java::security::cert::CertPathValidatorResult * validate(::java::security::cert::CertPath *, ::java::security::cert::CertPathParameters *);
private:
  static ::java::lang::String * CERT_PATH_VALIDATOR;
  ::java::security::cert::CertPathValidatorSpi * __attribute__((aligned(__alignof__( ::java::lang::Object)))) validatorSpi;
  ::java::security::Provider * provider;
  ::java::lang::String * algorithm;
public:
  static ::java::lang::Class class$;
};

#endif // __java_security_cert_CertPathValidator__
