
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_net_ssl_SSLSocketFactory$ErrorSocketFactory__
#define __javax_net_ssl_SSLSocketFactory$ErrorSocketFactory__

#pragma interface

#include <javax/net/ssl/SSLSocketFactory.h>
#include <gcj/array.h>

extern "Java"
{
  namespace java
  {
    namespace net
    {
        class InetAddress;
        class Socket;
    }
  }
  namespace javax
  {
    namespace net
    {
      namespace ssl
      {
          class SSLSocketFactory$ErrorSocketFactory;
      }
    }
  }
}

class javax::net::ssl::SSLSocketFactory$ErrorSocketFactory : public ::javax::net::ssl::SSLSocketFactory
{

public: // actually package-private
  SSLSocketFactory$ErrorSocketFactory(::java::lang::RuntimeException *);
public:
  ::java::net::Socket * createSocket();
  ::java::net::Socket * createSocket(::java::lang::String *, jint);
  ::java::net::Socket * createSocket(::java::lang::String *, jint, ::java::net::InetAddress *, jint);
  ::java::net::Socket * createSocket(::java::net::InetAddress *, jint);
  ::java::net::Socket * createSocket(::java::net::InetAddress *, jint, ::java::net::InetAddress *, jint);
  JArray< ::java::lang::String * > * getDefaultCipherSuites();
  JArray< ::java::lang::String * > * getSupportedCipherSuites();
  ::java::net::Socket * createSocket(::java::net::Socket *, ::java::lang::String *, jint, jboolean);
private:
  ::java::lang::RuntimeException * __attribute__((aligned(__alignof__( ::javax::net::ssl::SSLSocketFactory)))) x;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_net_ssl_SSLSocketFactory$ErrorSocketFactory__
