
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_plaf_basic_BasicSpinnerUI$DefaultLayoutManager__
#define __javax_swing_plaf_basic_BasicSpinnerUI$DefaultLayoutManager__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace java
  {
    namespace awt
    {
        class Component;
        class Container;
        class Dimension;
    }
  }
  namespace javax
  {
    namespace swing
    {
      namespace plaf
      {
        namespace basic
        {
            class BasicSpinnerUI;
            class BasicSpinnerUI$DefaultLayoutManager;
        }
      }
    }
  }
}

class javax::swing::plaf::basic::BasicSpinnerUI$DefaultLayoutManager : public ::java::lang::Object
{

  BasicSpinnerUI$DefaultLayoutManager(::javax::swing::plaf::basic::BasicSpinnerUI *);
public:
  virtual void layoutContainer(::java::awt::Container *);
  virtual ::java::awt::Dimension * minimumLayoutSize(::java::awt::Container *);
  virtual ::java::awt::Dimension * preferredLayoutSize(::java::awt::Container *);
  virtual void removeLayoutComponent(::java::awt::Component *);
  virtual void addLayoutComponent(::java::lang::String *, ::java::awt::Component *);
private:
  ::java::awt::Dimension * prefSize(::java::awt::Component *);
  void setBounds(::java::awt::Component *, jint, jint, jint, jint);
public: // actually package-private
  BasicSpinnerUI$DefaultLayoutManager(::javax::swing::plaf::basic::BasicSpinnerUI *, ::javax::swing::plaf::basic::BasicSpinnerUI$DefaultLayoutManager *);
private:
  ::java::awt::Component * __attribute__((aligned(__alignof__( ::java::lang::Object)))) editor;
  ::java::awt::Component * next;
  ::java::awt::Component * previous;
public: // actually package-private
  ::javax::swing::plaf::basic::BasicSpinnerUI * this$0;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_swing_plaf_basic_BasicSpinnerUI$DefaultLayoutManager__
