
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __org_xml_sax_helpers_ParserFactory__
#define __org_xml_sax_helpers_ParserFactory__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace org
  {
    namespace xml
    {
      namespace sax
      {
          class Parser;
        namespace helpers
        {
            class ParserFactory;
        }
      }
    }
  }
}

class org::xml::sax::helpers::ParserFactory : public ::java::lang::Object
{

  ParserFactory();
public:
  static ::org::xml::sax::Parser * makeParser();
  static ::org::xml::sax::Parser * makeParser(::java::lang::String *);
  static ::java::lang::Class class$;
};

#endif // __org_xml_sax_helpers_ParserFactory__
