
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_text_AsyncBoxView$ChildLocator__
#define __javax_swing_text_AsyncBoxView$ChildLocator__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace java
  {
    namespace awt
    {
        class Graphics;
        class Rectangle;
        class Shape;
    }
  }
  namespace javax
  {
    namespace swing
    {
      namespace text
      {
          class AsyncBoxView;
          class AsyncBoxView$ChildLocator;
          class AsyncBoxView$ChildState;
      }
    }
  }
}

class javax::swing::text::AsyncBoxView$ChildLocator : public ::java::lang::Object
{

public:
  AsyncBoxView$ChildLocator(::javax::swing::text::AsyncBoxView *);
  virtual void childChanged(::javax::swing::text::AsyncBoxView$ChildState *);
  virtual jint getViewIndexAtPoint(jfloat, jfloat, ::java::awt::Shape *);
  virtual ::java::awt::Shape * getChildAllocation(jint, ::java::awt::Shape *);
  virtual void paintChildren(::java::awt::Graphics *);
public: // actually protected
  virtual ::java::awt::Shape * getChildAllocation(jint);
  virtual void setAllocation(::java::awt::Shape *);
  virtual jint getViewIndexAtVisualOffset(jfloat);
private:
  jint updateChildOffsets(jfloat);
  void updateChildOffsetsToIndex(jint);
public: // actually protected
  ::javax::swing::text::AsyncBoxView$ChildState * __attribute__((aligned(__alignof__( ::java::lang::Object)))) lastValidOffset;
  ::java::awt::Rectangle * lastAlloc;
  ::java::awt::Rectangle * childAlloc;
public: // actually package-private
  ::javax::swing::text::AsyncBoxView * this$0;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_swing_text_AsyncBoxView$ChildLocator__
