
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __org_relaxng_datatype_ValidationContext__
#define __org_relaxng_datatype_ValidationContext__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace org
  {
    namespace relaxng
    {
      namespace datatype
      {
          class ValidationContext;
      }
    }
  }
}

class org::relaxng::datatype::ValidationContext : public ::java::lang::Object
{

public:
  virtual ::java::lang::String * resolveNamespacePrefix(::java::lang::String *) = 0;
  virtual ::java::lang::String * getBaseUri() = 0;
  virtual jboolean isUnparsedEntity(::java::lang::String *) = 0;
  virtual jboolean isNotation(::java::lang::String *) = 0;
  static ::java::lang::Class class$;
} __attribute__ ((java_interface));

#endif // __org_relaxng_datatype_ValidationContext__
