
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_colorchooser_DefaultHSBChooserPanel$MainGradientMouseListener__
#define __javax_swing_colorchooser_DefaultHSBChooserPanel$MainGradientMouseListener__

#pragma interface

#include <java/awt/event/MouseAdapter.h>
extern "Java"
{
  namespace java
  {
    namespace awt
    {
        class Point;
      namespace event
      {
          class MouseEvent;
      }
    }
  }
  namespace javax
  {
    namespace swing
    {
      namespace colorchooser
      {
          class DefaultHSBChooserPanel;
          class DefaultHSBChooserPanel$MainGradientMouseListener;
      }
    }
  }
}

class javax::swing::colorchooser::DefaultHSBChooserPanel$MainGradientMouseListener : public ::java::awt::event::MouseAdapter
{

public: // actually package-private
  DefaultHSBChooserPanel$MainGradientMouseListener(::javax::swing::colorchooser::DefaultHSBChooserPanel *);
public:
  virtual void mousePressed(::java::awt::event::MouseEvent *);
  virtual void mouseDragged(::java::awt::event::MouseEvent *);
  virtual void mouseMoved(::java::awt::event::MouseEvent *);
private:
  void update(::java::awt::Point *);
  void updateH(::java::awt::Point *);
  void updateS(::java::awt::Point *);
  void updateB(::java::awt::Point *);
public: // actually package-private
  ::javax::swing::colorchooser::DefaultHSBChooserPanel * __attribute__((aligned(__alignof__( ::java::awt::event::MouseAdapter)))) this$0;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_swing_colorchooser_DefaultHSBChooserPanel$MainGradientMouseListener__
