
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_JPopupMenu$Separator__
#define __javax_swing_JPopupMenu$Separator__

#pragma interface

#include <javax/swing/JSeparator.h>
extern "Java"
{
  namespace javax
  {
    namespace swing
    {
        class JPopupMenu$Separator;
    }
  }
}

class javax::swing::JPopupMenu$Separator : public ::javax::swing::JSeparator
{

public:
  JPopupMenu$Separator();
  virtual ::java::lang::String * getUIClassID();
  static ::java::lang::Class class$;
};

#endif // __javax_swing_JPopupMenu$Separator__
