
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_print_attribute_standard_MediaPrintableArea__
#define __javax_print_attribute_standard_MediaPrintableArea__

#pragma interface

#include <java/lang/Object.h>
#include <gcj/array.h>

extern "Java"
{
  namespace javax
  {
    namespace print
    {
      namespace attribute
      {
        namespace standard
        {
            class MediaPrintableArea;
        }
      }
    }
  }
}

class javax::print::attribute::standard::MediaPrintableArea : public ::java::lang::Object
{

public:
  MediaPrintableArea(jfloat, jfloat, jfloat, jfloat, jint);
  MediaPrintableArea(jint, jint, jint, jint, jint);
  ::java::lang::Class * getCategory();
  ::java::lang::String * getName();
  jfloat getHeight(jint);
  jfloat getWidth(jint);
  jfloat getX(jint);
  jfloat getY(jint);
  jboolean equals(::java::lang::Object *);
  ::java::lang::String * toString();
  jint hashCode();
  ::java::lang::String * toString(jint, ::java::lang::String *);
  JArray< jfloat > * getPrintableArea(jint);
private:
  static const jlong serialVersionUID = -1597171464050795793LL;
public:
  static const jint INCH = 25400;
  static const jint MM = 1000;
private:
  jint __attribute__((aligned(__alignof__( ::java::lang::Object)))) x;
  jint y;
  jint w;
  jint h;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_print_attribute_standard_MediaPrintableArea__
