
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __org_xml_sax_ext_DeclHandler__
#define __org_xml_sax_ext_DeclHandler__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace org
  {
    namespace xml
    {
      namespace sax
      {
        namespace ext
        {
            class DeclHandler;
        }
      }
    }
  }
}

class org::xml::sax::ext::DeclHandler : public ::java::lang::Object
{

public:
  virtual void elementDecl(::java::lang::String *, ::java::lang::String *) = 0;
  virtual void attributeDecl(::java::lang::String *, ::java::lang::String *, ::java::lang::String *, ::java::lang::String *, ::java::lang::String *) = 0;
  virtual void internalEntityDecl(::java::lang::String *, ::java::lang::String *) = 0;
  virtual void externalEntityDecl(::java::lang::String *, ::java::lang::String *, ::java::lang::String *) = 0;
  static ::java::lang::Class class$;
} __attribute__ ((java_interface));

#endif // __org_xml_sax_ext_DeclHandler__
